package de.intevation.flys.artifacts.model;

import de.intevation.flys.utils.DoubleUtil;

import gnu.trove.TDoubleArrayList;

import org.apache.log4j.Logger;


/**
 * This class represents a pool of data triples that consists of 'W', 'Q' and
 * 'KM' data.
 *
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class WQKms
extends      WQ
implements   WKms, QKms
{
    private static Logger logger = Logger.getLogger(WQKms.class);

    /** The array that contains the 'KMs' values. */
    protected TDoubleArrayList kms;


    public WQKms() {
        this("");
    }


    public WQKms(String name) {
        super(name);
        this.kms = new TDoubleArrayList();
    }


    public WQKms(int capacity) {
        this(capacity, "");
    }


    public WQKms(int capacity, String name) {
        super(capacity, name);
        this.kms = new TDoubleArrayList(capacity);
    }

    public WQKms(double [] kms, double [] qs, double [] ws) {
        this(kms, qs, ws, "");
    }


    public WQKms(double [] kms, double [] qs, double [] ws, String name) {
        super(qs, ws, name);
        this.kms = new TDoubleArrayList(kms);
    }

    @Override
    public void removeNaNs() {
        DoubleUtil.removeNaNs(new TDoubleArrayList [] { ws, qs, kms });
    }

    /**
     * Adds a new row to this data pool.
     *
     * @param w a W.
     * @param q a Q.
     * @param km a kms.
     */
    public void add(double w, double q, double km) {
        super.add(w, q);
        kms.add(km);
    }

    @Override
    public double [] get(int idx) {
        return get(idx, new double [3]);
    }

    /**
     * This method returns a triple of W, Q and Kms in a single 3dim array.
     *
     * @param idx The position of the triple.
     * @param dst destination array
     *
     * @return a triple of [W, Q, Kms] in dst.
     */
    @Override
    public double [] get(int idx, double [] dst) {
        dst[0] = ws .getQuick(idx);
        dst[1] = qs .getQuick(idx);
        dst[2] = kms.getQuick(idx);
        return dst;
    }

    @Override
    public double getKm(int idx) {
        return kms.getQuick(idx);
    }

    @Override
    public TDoubleArrayList allKms() {
        return kms;
    }

    @Override
    public TDoubleArrayList allWs() {
        return ws;
    }

    @Override
    public TDoubleArrayList allQs() {
        return qs;
    }

    public double[] getKms() {
        return kms.toNativeArray();
    }

    /**
     * Returns a string that consist of the first and last kilometer.
     *
     * @return a string that consist of the first and last kilometer.
     */
    public String toString() {
        double from = getKm(0);
        double to   = getKm(size()-1);
        return from + " - " + to;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
